package org.processmining.dataawarereplayer.precision;

import java.util.HashMap;
import java.util.Map;

import org.deckfour.xes.classification.XEventClassifier;
import org.processmining.dataawarereplayer.precision.PrecisionConfigPanel.PrecisionType;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;

import com.google.common.collect.HashMultimap;
import com.google.common.collect.SetMultimap;

public class PrecisionConfig {

	private final Marking initialMarking;
	private final SetMultimap<String, Transition> activityMapping; 
	private final Map<String, String> variableMapping;
	private final XEventClassifier classifier;
	
	private boolean debug = false;
	private boolean penalizeImpreciseEnding = false;
	private boolean assumeUnassignedVariablesFree = true;
	private int concurrentThreads = Math.max(1, Runtime.getRuntime().availableProcessors());

	@SuppressWarnings("rawtypes")
	private Map<String, Class> extraAttributes = new HashMap<>();

	private SetMultimap<Transition, String> executors = HashMultimap.create();
	private PrecisionType precisionMeasure = PrecisionType.ACTIVITY;	;

	public PrecisionConfig(Marking initialMarking, SetMultimap<String, Transition> activityMapping, XEventClassifier classifier, Map<String, String> variableMapping) {
		this.initialMarking = initialMarking;
		this.activityMapping = activityMapping;
		this.classifier = classifier;
		this.variableMapping = variableMapping;
	}
	
	public Marking getInitialMarking() {
		return initialMarking;
	}

	public boolean isDebug() {
		return debug;
	}

	public boolean isPenalizeImpreciseEnding() {
		return penalizeImpreciseEnding;
	}

	public void setDebug(boolean isDebug) {
		this.debug = isDebug;
	}

	public void setPenalizeImpreciseEnding(boolean isPenalizeImpreciseEnding) {
		this.penalizeImpreciseEnding = isPenalizeImpreciseEnding;
	}

	public void setConcurrentThreads(int concurrentThreads) {
		this.concurrentThreads = concurrentThreads;
	}
	
	public int getConcurrentThreads() {
		return concurrentThreads;
	}

	@SuppressWarnings("rawtypes")
	public void setExtraAttributes(Map<String, Class> extraAttributes) {
		this.extraAttributes = extraAttributes;
	}
	
	@SuppressWarnings("rawtypes")
	public Map<String, Class> getExtraAttributes() {
		return extraAttributes;
	}

	public void setExecutors(SetMultimap<Transition, String> executors) {
		this.executors = executors;
	}

	public SetMultimap<Transition, String> getExecutors() {
		return executors;
	}

	public void setPrecisionMeasure(PrecisionType precisionMeasure) {
		this.precisionMeasure = precisionMeasure;
	}

	public PrecisionType getPrecisionMeasure() {
		return precisionMeasure;
	}

	public Map<String, String> getVariableMapping() {
		return variableMapping;
	}

	public XEventClassifier getClassifier() {
		return classifier;
	}

	public SetMultimap<String, Transition> getActivityMapping() {
		return activityMapping;
	}

	public boolean isAssumeUnassignedVariablesFree() {
		return assumeUnassignedVariablesFree;
	}

	public void setAssumeUnassignedVariablesFree(boolean assumeUnassignedVariablesFree) {
		this.assumeUnassignedVariablesFree = assumeUnassignedVariablesFree;
	}

}
