package org.processmining.dataawarereplayer.precision.projection;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.extension.std.XOrganizationalExtension;
import org.deckfour.xes.model.XLog;
import org.processmining.datapetrinets.DataPetriNet;
import org.processmining.datapetrinets.expression.GuardExpression;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.DataElement;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.PNWDTransition;
import org.processmining.plugins.balancedconformance.mapping.StringDiscretizer;
import org.processmining.plugins.balancedconformance.mapping.StringDiscretizerException;
import org.processmining.plugins.balancedconformance.mapping.StringDiscretizerImpl;

import com.google.common.collect.ImmutableSetMultimap;
import com.google.common.collect.SetMultimap;
import com.google.common.collect.Sets;

class LogMapping {

	private final StringDiscretizer discretizer;
	@SuppressWarnings("rawtypes")
	private final Map<String, Class> variableType;
	@SuppressWarnings("rawtypes")
	private final Map<String, Class> extraAttributes;
	private final Map<String, String> variableMappingWithExtraAttributes;
	private final SetMultimap<String, Transition> eventIdentityToTransition;
	private final XEventClasses eventClasses;
	private final Map<Transition, LinkedHashSet<String>> variablesToWrite;
	private final Map<String, Transition> localIdToTransition;

	public LogMapping(DataPetriNet net, XLog log, XEventClasses eventClasses,
			SetMultimap<String, Transition> eventIdentityToTransition, Map<String, String> variableMapping,
			@SuppressWarnings("rawtypes") Map<String, Class> extraAttributes, SetMultimap<Transition, String> executors)
			throws StringDiscretizerException {
		super();
		this.eventClasses = eventClasses;
		this.eventIdentityToTransition = eventIdentityToTransition;
		this.extraAttributes = extraAttributes;

		this.variableType = new HashMap<>();
		for (DataElement variable : net.getVariables()) {
			variableType.put(variable.getVarName(), variable.getType());
		}
		// Add the extra attributes
		variableType.putAll(extraAttributes);

		variablesToWrite = new HashMap<>();

		Map<Transition, LinkedHashSet<String>> attributeMapForDiscretizer = new HashMap<>();
		List<GuardExpression> guards = new ArrayList<>();

		for (Transition transition : net.getTransitions()) {
			if (transition instanceof PNWDTransition) {
				PNWDTransition pnwdTransition = (PNWDTransition) transition;
				if (pnwdTransition.getGuardExpression() != null) {
					guards.add(pnwdTransition.getGuardExpression());
				}
				LinkedHashSet<String> writeOps = new LinkedHashSet<>();
				for (DataElement writeOp : pnwdTransition.getWriteOperations()) {
					writeOps.add(writeOp.getVarName());
				}
				variablesToWrite.put(transition, writeOps);
				attributeMapForDiscretizer.put(transition,
						Sets.newLinkedHashSet(Sets.union(writeOps, extraAttributes.keySet())));
			} else {
				variablesToWrite.put(transition, Sets.<String>newLinkedHashSet());
				attributeMapForDiscretizer.put(transition, Sets.<String>newLinkedHashSet());
			}
		}

		this.variableMappingWithExtraAttributes = new HashMap<>();
		for (String attribute : extraAttributes.keySet()) {
			this.variableMappingWithExtraAttributes.put(attribute, attribute);
		}
		this.variableMappingWithExtraAttributes.putAll(variableMapping);

		SetMultimap<String, String> extraValues = ImmutableSetMultimap.<String, String>builder()
				.putAll(XOrganizationalExtension.KEY_RESOURCE, executors.values()).build();

		discretizer = new StringDiscretizerImpl(log, guards, variableMappingWithExtraAttributes, variableType,
				Collections.<String, Object>emptyMap(), eventIdentityToTransition, attributeMapForDiscretizer,
				eventClasses.getClassifier(), extraValues);

		localIdToTransition = new HashMap<String, Transition>();
		for (Transition t : net.getTransitions()) {
			localIdToTransition.put(t.getLocalID().toString(), t);
		}
	}

	public StringDiscretizer getDiscretizer() {
		return discretizer;
	}

	@SuppressWarnings("rawtypes")
	public Map<String, Class> getAttributeType() {
		return variableType;
	}

	public Set<String> getExtraAttributes() {
		return extraAttributes.keySet();
	}

	public Map<String, String> getVariableMapping() {
		return variableMappingWithExtraAttributes;
	}

	public SetMultimap<String, Transition> getEventIdentifyToTransition() {
		return eventIdentityToTransition;
	}

	public XEventClasses getEventClasses() {
		return eventClasses;
	}

	public Map<Transition, LinkedHashSet<String>> getVariablesToWrite() {
		return variablesToWrite;
	}

	public Map<String, Transition> getLocalIdToTransition() {
		return localIdToTransition;
	}

}
