package org.processmining.plugins.DataConformance.DCnet;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import org.processmining.models.DCNet.DCNetNode;
import org.processmining.models.cnet.CNetNode;

public class DCState implements Cloneable
{
	private HashMap<DCNetNode,LinkedList<CNetNode>> obligations=new HashMap<DCNetNode, LinkedList<CNetNode>>();
	
	private HashMap<String,Object> dataVariables=new HashMap<String, Object>();
	
	@SuppressWarnings("unchecked")
	public Object clone()
	{
		DCState clone=null;
		try 
		{
			clone = (DCState) super.clone();
			//Cloning of the obligations' map
			clone.obligations=(HashMap<DCNetNode, LinkedList<CNetNode>>) obligations.clone();
			//The list of obligations per node still needs to be cloned
			for(Entry<DCNetNode, LinkedList<CNetNode>> obligationEntry : obligations.entrySet())
			{
				obligationEntry.setValue((LinkedList<CNetNode>) obligationEntry.getValue().clone());
			}
			
			//Cloning of the map of data variables
			clone.dataVariables=(HashMap<String, Object>) dataVariables.clone();
		} 
		catch (CloneNotSupportedException e) {
			e.printStackTrace();
		}
		return clone;
	}
	
	public void addObligation(CNetNode activityToPerform, CNetNode preceedingActivity) 
	{
		LinkedList<CNetNode> obligationActivity = obligations.get(activityToPerform);
		if (obligationActivity == null)
		{
			obligationActivity = new LinkedList<CNetNode>();
			obligations.put((DCNetNode)activityToPerform, obligationActivity);
		}
		obligationActivity.add(preceedingActivity);
	}
	
	public Set<DCNetNode> getPerformableActivities()
	{
		return Collections.unmodifiableSet(obligations.keySet());
	}
	
	public List<CNetNode> getObligationsPerActivity(CNetNode node)
	{
		List<CNetNode> obligationsForNode = obligations.get(node);
		if (obligationsForNode == null)
			return null;
		else
		{
			assert(obligationsForNode.size()>0);
			return Collections.unmodifiableList(obligationsForNode);
		}
	}
	
	public void removeObligations(CNetNode node, Collection<CNetNode> nodeList)
	{
		List<CNetNode> obligationsForNode = obligations.get(node);
		HashSet<CNetNode> f = new HashSet<CNetNode>();
		f.addAll(nodeList);
		if (obligationsForNode != null)
		{
			ListIterator<CNetNode> iter=obligationsForNode.listIterator();
			while(iter.hasNext() && f.size()>0)
			{
				CNetNode next=iter.next();
				if(f.contains(next))
				{
					iter.remove();
					f.remove(next);
				}
			}
		}
		if (obligationsForNode.size()==0)
		{
			obligations.remove(node);
		}
	}

	public void addObligations(Collection<CNetNode> outputBinding, CNetNode node) 
	{
		for(CNetNode activityToPerform : outputBinding)
			if (!activityToPerform.equals(DCNetNode.emptyNode))
				addObligation(activityToPerform, node);
	}

	public Map<String, Object> getDataVariables() 
	{
		return Collections.unmodifiableMap(dataVariables);
	}
	
	public void setDataVariable(String varName, Object value)
	{
		assert(value!=null);
		dataVariables.put(varName, value);
	}

	public void setDataVariables(Map<String, Object> writtenVariables) {
		dataVariables.putAll(writtenVariables);
		
	}
	
	public String toString()
	{
		return obligations+"; "+dataVariables;
	}

	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((dataVariables == null) ? 0 : dataVariables.hashCode());
		result = prime * result + ((obligations == null) ? 0 : obligations.hashCode());
		return result;
	}

	public boolean equals(Object obj) {
		if (getClass() != obj.getClass())
			return false;
		DCState other = (DCState) obj;
		return (dataVariables.equals(other.dataVariables) && obligations.equals(other.obligations));
	}
	
}
