package org.processmining.plugins.DataConformance.DataAlignment;

import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.datapetrinets.expression.GuardExpression;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.DataElement;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.PNWDTransition;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.plugins.petrinet.replayresult.StepTypes;

import gnu.trove.map.hash.THashMap;
import gnu.trove.set.hash.THashSet;

public class ControlFlowAlignmentBridge {

	/**
	 * Variable Mapping that is just read!
	 */
	private final Map<String, String> variableMapping;
	@SuppressWarnings("rawtypes")
	private final Map<String, Class> varType;
	private final TransEvClassMapping activityMapping;

	public ControlFlowAlignmentBridge(XLog log, Map<String, String> variableMapping,
			@SuppressWarnings("rawtypes") Map<String, Class> varType, TransEvClassMapping activityMapping) {
		super();
		this.variableMapping = variableMapping;
		this.varType = varType;
		this.activityMapping = activityMapping;
	}

	public IControlFlowAlignment<Object, Object> buildControlFlowAlignment(List<Object> nodeInstances,
			List<StepTypes> stepTypes, XTrace xTrace, int traceIndex, float controlFlowFitness) {

		Iterator<XEvent> eventIter = xTrace.iterator();
		Iterator<Object> transIter = nodeInstances.iterator();
		Transition transition = null;
		XEvent nextEvent = null;
		int stepType = -1;
		ControlFlowAlignment retValue = new ControlFlowAlignment(getTraceName(xTrace, traceIndex), controlFlowFitness);
		for (StepTypes step : stepTypes) {
			switch (step) {
				case LMGOOD :
				case LMNOGOOD :
					nextEvent = eventIter.next();
					transition = (Transition) transIter.next();
					stepType = ControlFlowAlignmentStep.MOVE_IN_BOTH;
					break;
				case L :
					nextEvent = eventIter.next();
					transition = null;
					transIter.next();
					stepType = ControlFlowAlignmentStep.MOVE_IN_LOG;
					break;
				case MINVI :
					stepType = ControlFlowAlignmentStep.MOVE_IN_MODEL_INVISIBLE;
					nextEvent = null;
					transition = (Transition) transIter.next();
					break;
				case MREAL :
					nextEvent = null;
					transition = (Transition) transIter.next();
					stepType = ControlFlowAlignmentStep.MOVE_IN_MODEL_VISIBLE;
					break;
				default :
					throw new IllegalArgumentException("Inexpected step type " + step
							+ ". Expected move in both, in log or (in)visible in process.");
			}
			String activityName;
			if (transition != null)
				activityName = getTransitionLabel(transition);
			else
				activityName = activityMapping.getEventClassifier().getClassIdentity(nextEvent);
			GuardExpression expr;
			if (transition != null && transition instanceof PNWDTransition)
				expr = ((PNWDTransition) transition).getGuardExpression();
			else
				expr = null;
			Map<String, Object> varAssignments = getVariableAssignment(nextEvent, varType);
			Set<String> variablesToWrite = getVariabletoWrite(transition);
			retValue.add(new ControlFlowAlignmentStep(activityName, expr, varAssignments, stepType, variablesToWrite));
		}
		return retValue;
	}

	/**
	 * Avoiding the call to {@link Transition#getLabel()}, which involves a
	 * synchronised block
	 * 
	 * @param transition
	 * @return
	 */
	private String getTransitionLabel(Transition transition) {
		return transition.getLabel();
	}

	/**
	 * Avoiding the call to
	 * {@link XConceptExtension#extractName(org.deckfour.xes.model.XAttributable)}
	 * 
	 * @param xTrace
	 * @param traceIndex
	 * @return
	 */
	private String getTraceName(XTrace xTrace, int traceIndex) {
		String traceName = XConceptExtension.instance().extractName(xTrace);
		return traceName == null ? String.valueOf(traceIndex) : traceName;
	}

	private Set<String> getVariabletoWrite(Transition transition) {
		if (transition == null)
			return null;
		Set<String> varToWrite = new THashSet<String>();
		if (transition instanceof PNWDTransition) {
			for (DataElement elem : ((PNWDTransition) transition).getWriteOperations())
				varToWrite.add(elem.getVarName());
		}
		return varToWrite;
	}

	private Map<String, Object> getVariableAssignment(XEvent nextEvent,
			@SuppressWarnings("rawtypes") Map<String, Class> varType) {
		Map<String, Object> variableAssignment = new THashMap<String, Object>();
		if (nextEvent != null)
			for (Entry<String, XAttribute> attrib : nextEvent.getAttributes().entrySet()) {
				String attrName = null;
				for (Entry<String, String> mapping : variableMapping.entrySet()) {
					if (mapping.getValue().equals(attrib.getKey())) {
						attrName = mapping.getKey();
						break;
					}
				}
				if (attrName != null) {
					Object value = org.processmining.plugins.DataConformance.Utility.getValue(attrib.getValue(),
							varType.get(attrName));
					variableAssignment.put(attrName, value);
				}
			}
		return variableAssignment;
	}

}
