package org.processmining.plugins.DataConformance.DataAlignment;

import java.text.ParseException;
import java.util.Map;
import java.util.Set;

import org.processmining.datapetrinets.expression.GuardExpression;
import org.processmining.models.guards.Expression;

import com.google.common.base.Joiner;

public class ControlFlowAlignmentStep implements IControlFlowAlignmentStep<Object, Object> {

	private final String activityName;

	private final Expression expression;
	private final Map<String, Object> variableAssignments;
	private final int moveType;
	private final Set<String> variablesToWrite;

	private final Object processObject;
	private Object logObject;

	/**
	 * Please use the constructor
	 * {@link #ControlFlowAlignmentStep(String, GuardExpression, Map, int, Set)}
	 * instead!
	 */
	@Deprecated
	public ControlFlowAlignmentStep(String activityName, Expression oldExpression, Map<String, Object> varAssignments,
			int moveType, Set<String> variablesToWrite) {
		super();
		this.processObject = null;
		this.logObject = null;
		this.activityName = activityName;
		this.expression = oldExpression;
		
		if (varAssignments != null) {
			this.variableAssignments = varAssignments;
		} else {
			this.variableAssignments = null;
		}
		if (variablesToWrite != null) {
			this.variablesToWrite = variablesToWrite;
		} else {
			this.variablesToWrite = null;
		}
		this.moveType = moveType;
	}

	public ControlFlowAlignmentStep(String activityName, GuardExpression expr, Map<String, Object> varAssignments,
			int moveType, Set<String> variablesToWrite) {
		this(null, null, activityName, expr, varAssignments, moveType, variablesToWrite);
	}

	public ControlFlowAlignmentStep(Object processObject, Object logObject, String activityName, GuardExpression expr,
			Map<String, Object> varAssignments, int moveType, Set<String> variablesToWrite) {
		super();
		this.processObject = processObject;
		this.logObject = logObject;
		this.activityName = activityName;
		try {
			this.expression = new Expression(expr == null ? "true" : expr.toCanonicalString());
		} catch (ParseException e) {
			throw new RuntimeException();
		}
		if (varAssignments != null) {
			this.variableAssignments = varAssignments;
		} else {
			this.variableAssignments = null;
		}
		if (variablesToWrite != null) {
			this.variablesToWrite = variablesToWrite;
		} else {
			this.variablesToWrite = null;
		}
		this.moveType = moveType;
	}

	public String getActivityName() {
		return activityName;
	}

	public int getMoveType() {
		return moveType;
	}

	public Map<String, Object> getVariableAssignments() {
		return variableAssignments;
	}

	public Set<String> getVariableToWrite() {
		return variablesToWrite;
	}

	public String toString() {
		return activityName + "(" + moveType + ")" + " Writes[" + Joiner.on(',').join(variablesToWrite) + "]"
				+ " Values[" + Joiner.on(',').join(variableAssignments.entrySet()) + "]";
	}

	public Object getProcessObject() {
		return processObject;
	}

	public Object getLogObject() {
		return logObject;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.processmining.plugins.DataConformance.DataAlignment.
	 * IControlFlowAlignmentStep#getGuardExpression()
	 */
	@Deprecated
	public Expression getGuardExpression() {
		return expression;
	}

}
