package org.processmining.plugins.DataConformance.DataAlignment.DCNet;

import gnu.trove.iterator.TIntIterator;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.Map.Entry;
import java.util.regex.Matcher;

import net.sf.javailp.Constraint;
import net.sf.javailp.Linear;
import net.sf.javailp.Problem;

import org.processmining.models.DCNet.ActivityBinding;
import org.processmining.models.DCNet.DCNet;
import org.processmining.models.DCNet.DCNetBinding;
import org.processmining.models.DCNet.DCNetNode;
import org.processmining.models.guards.NumericValueConversion;
import org.processmining.plugins.cnet.replayresult.CNetRepResult;
import org.processmining.plugins.cnet.replayresult.CNetSyncReplayResult;

public class Condition
{
	public final Problem ilpProblem=new Problem();
	public final List<HashMap<String,String>>  varWrittens=new ArrayList<HashMap<String,String>>();
	public final List<ActivityBinding> activitySet;
	public boolean[] logMove;
	private static String SPECIAL_CHAR=Matcher.quoteReplacement("W___");
	private final float controlFlowFitness;
	
	public Condition(List<ActivityBinding> activitySet, boolean[] logMove, float controlFlowFitness) {
		this.activitySet=activitySet;
		this.logMove=logMove;
		this.controlFlowFitness=controlFlowFitness;
	}


	public static Map<Integer,Condition> getConditions(DCNet net, CNetRepResult input) throws ParseException
	{
		HashMap<Integer,Condition> retValue=new HashMap<Integer, Condition>();
		for (CNetSyncReplayResult output : input)
		{
			short moves[]=output.getModelMoveBinding();
			TIntIterator traceIndexIterator=output.getTraceIndex().iterator();
			ArrayList<ActivityBinding> list=new ArrayList<ActivityBinding>(moves.length/2);
			for(int i=0;i<moves.length;i+=2)
			{
				ActivityBinding newActBinding=new ActivityBinding();

				DCNetBinding inBinding=(DCNetBinding) input.getBindingOfEnc(moves[i]);
				DCNetBinding outBinding=(DCNetBinding) input.getBindingOfEnc(moves[i+1]);
				DCNetNode node=null;
				if (inBinding!=null)
					node=(DCNetNode) inBinding.getNode();
				else if (outBinding!=null)
					node=(DCNetNode) outBinding.getNode();
				newActBinding.inputBinding=inBinding;
				newActBinding.outputBinding=outBinding;
				newActBinding.node=node;
				list.add(newActBinding);
			}
			Condition cond=Condition.getCondition(net, list, output);
			while(traceIndexIterator.hasNext())
				retValue.put(traceIndexIterator.next(), cond);
		}
		return retValue;
	}
	
	private static Condition getCondition(DCNet net, List<ActivityBinding> activitySet, CNetSyncReplayResult alignment) throws ParseException
	{
		HashMap<String,Integer> timesWrittenVariable=new HashMap<String, Integer>();
		float controlFlowFitness=alignment.getInfo().get(CNetRepResult.TRACEFITNESS).floatValue();
		Condition retValue=new Condition(activitySet,alignment.getLogMove(),controlFlowFitness);
		HashMap<String,String> listVariableWritten; 
		//Add constraints to the problem
		for(ActivityBinding binding : activitySet)
		{
			if (binding.node == null)
				continue;

			listVariableWritten = new HashMap<String, String>();
			String enablingConditions=binding.node.getGuard();
			if (binding.inputBinding!=null)
				enablingConditions+=" && "+binding.inputBinding.getDataExpression();
			if (binding.outputBinding!=null)
				enablingConditions+=" && "+binding.outputBinding.getDataExpression();			
			Set<String> varWritten=DCNet.getWrittenVariables(enablingConditions);
			for(String var : varWritten)
			{
				Integer times=timesWrittenVariable.get(var);
				if (times==null)
					times=1;
				else
					times++;
				timesWrittenVariable.put(var, times);
				listVariableWritten.put(var, var+times);
			}
			retValue.varWrittens.add(listVariableWritten);
			String expressions[]=new String[3];
			String quoteReplacement=Matcher.quoteReplacement(DCNet.OUTPUTVAR_IDENTIFIER+"");
			if(binding.inputBinding!=null)
				expressions[0]=binding.inputBinding.getDataExpression().replaceAll(quoteReplacement, SPECIAL_CHAR);
			expressions[1]=binding.node.getGuard().replaceAll(quoteReplacement, SPECIAL_CHAR);
			if(binding.outputBinding!=null)
				expressions[2]=binding.outputBinding.getDataExpression().replaceAll(quoteReplacement, SPECIAL_CHAR);
			for(String expr2 : expressions)
			{
				if (expr2==null)
					continue;
				StringTokenizer tokenizer=new StringTokenizer(expr2,"&");
				while(tokenizer.hasMoreElements())
				{
					String subExpr=tokenizer.nextToken();
					StringTokenizer stringConstantTokenizer = new StringTokenizer(subExpr,"\"",true);
					StringBuffer outputString=new StringBuffer();
					boolean toConvert=false;
					String stringToConvert="";
					while(stringConstantTokenizer.hasMoreTokens())
					{
						String nextToken=stringConstantTokenizer.nextToken();
						if (toConvert)
						{
							if (nextToken.equals("\""))
							{
								outputString.append(org.processmining.models.guards.NumericValueConversion.toNumericValue(stringToConvert));
								toConvert=false;
							}
							else
								stringToConvert=nextToken;
						}
						else
						{
						if (nextToken.equals("\""))
							toConvert=true;
						else
							outputString.append(nextToken);
						}
					}
					if (toConvert)
						throw new ParseException("Missing end string quote", -1);
					Expression e = new Expression(outputString.toString());
					if (e.getOperand2()==null)
						continue;
					Hashtable linearForm=e.generateLinearForm();
					if (linearForm==null)
						throw new ParseException("The expression is neither parseable nor linear", -1);
					Iterator<Entry> variables = linearForm.entrySet().iterator();
					Linear constraint=new Linear();
					String operator=null;
					Double constant=0D;
					while(variables.hasNext())
					{
						Entry nextEntry=variables.next();
						String nextVar=(String) nextEntry.getKey();
						Object nextVarValue=nextEntry.getValue();
						if (nextVar.equals("OPERATOR"))
						{
							operator=(String) nextEntry.getValue();
							continue;
						}
						if (nextVar.equals("CONSTANT")) {
							constant=((Number) nextEntry.getValue()).doubleValue();
							continue;
						}
						if (nextVar.contains(SPECIAL_CHAR))
						{
							nextVar=nextVar.replaceAll(SPECIAL_CHAR, " ").trim();
							nextVar+=timesWrittenVariable.get(nextVar);
						}
						else
						{
							if (varWritten.contains(nextVar))
								nextVar+=(timesWrittenVariable.get(nextVar)-1);	
							else
								nextVar+=timesWrittenVariable.get(nextVar);
						}
						if (nextVarValue instanceof String)
						{
							constraint.add(org.processmining.models.guards.NumericValueConversion.toNumericValue(nextVarValue),nextVar);	
						}
						else
							constraint.add((Number) nextVarValue,nextVar);

					}
					operator=operator.trim();
					if (operator.equals("=="))
						retValue.ilpProblem.add(constraint, "=", constant*(-1.0));
					else if (operator.equals("<") || operator.equals(">"))
					{
						operator+="=";
						retValue.ilpProblem.add(constraint, operator, constant*(-1.0)-0.001);
					}
					else if (operator.equals("<>") || operator.equals("!="))
					{
						
						retValue.ilpProblem.add(constraint, "<=", constant*(-1.0)-0.001);
						retValue.ilpProblem.add(constraint, ">=", constant*(-1.0)+0.001);					
					}
					else
						retValue.ilpProblem.add(constraint, operator, constant*(-1.0));
				}
			}

		}
		for(String varName : net.getVariables())
		{
			Comparable maxValue=Collections.max(net.getSetValues(net.getSetForVariable(varName)));
			Comparable minValue=Collections.min(net.getSetValues(net.getSetForVariable(varName)));			
			Class type;
			if (maxValue instanceof String)
			{
				type=Double.class;
				maxValue=NumericValueConversion.toNumericValue(maxValue);
			}
			else if (maxValue instanceof Short)
				type=Integer.class;
			else if (maxValue instanceof Float)
				type=Double.class;
			else
				type=maxValue.getClass();
			if (minValue instanceof String)
			{
				type=Double.class;
				minValue=org.processmining.models.guards.NumericValueConversion.toNumericValue(maxValue);
			}
			else if (minValue instanceof Short)
				type=Integer.class;
			else if (minValue instanceof Float)
				type=Double.class;
			else
				type=minValue.getClass();

			if (timesWrittenVariable.get(varName) != null)
				for(int i=1;i<=timesWrittenVariable.get(varName);i++)
				{
					retValue.ilpProblem.setVarType(varName+i, type);
					if (maxValue instanceof Number)
						retValue.ilpProblem.setVarUpperBound(varName+i, (Number) maxValue);
					if (minValue instanceof Number)
						retValue.ilpProblem.setVarLowerBound(varName+i, (Number) minValue);
				}
		}
		return retValue;
	}


	public Problem getIlpProblem(boolean sharedMemory) {
		if (sharedMemory)
		{
			return ilpProblem;
		}
		Problem clone=new Problem();
		//clone variables
		for(Object variable : ilpProblem.getVariables())
		{
			clone.setVarType(variable, ilpProblem.getVarType(variable));
			clone.setVarUpperBound(variable, ilpProblem.getVarUpperBound(variable));
		}
		//clone constraint
		for(Constraint costr: ilpProblem.getConstraints())
			clone.add(costr);
		return clone;
	}


	public List<HashMap<String, String>> getVarWrittens() {
		return varWrittens;
	}


	public float getControlFlowFitness() {
		return controlFlowFitness;
	}


}
