package org.processmining.plugins.DataConformance.DataAlignment.PetriNet;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.TreeSet;

import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.datapetrinets.DataPetriNet;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.Progress;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.models.connections.petrinets.EvClassLogPetrinetConnection;
import org.processmining.models.connections.petrinets.behavioral.FinalMarkingConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.DataElement;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.DataConformance.DataAlignment.ControlFlowAlignment;
import org.processmining.plugins.DataConformance.DataAlignment.ControlFlowAlignmentBridge;
import org.processmining.plugins.DataConformance.DataAlignment.DataAlignerBuilder;
import org.processmining.plugins.DataConformance.DataAlignment.DataAlignerListener;
import org.processmining.plugins.DataConformance.DataAlignment.DataAlignmentState;
import org.processmining.plugins.DataConformance.GUI.MappingPanel;
import org.processmining.plugins.DataConformance.GUI.VariableMatchCostPanel;
import org.processmining.plugins.DataConformance.framework.VariableMatchCosts;
import org.processmining.plugins.astar.petrinet.PetrinetReplayerWithoutILP;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.plugins.petrinet.finalmarkingprovider.MarkingEditorPanel;
import org.processmining.plugins.petrinet.replayer.PNLogReplayer;
import org.processmining.plugins.petrinet.replayer.algorithms.costbasedcomplete.CostBasedCompleteParam;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.replayer.replayresult.SyncReplayResult;

import nl.tue.astar.AStarException;

public class DataAlignment implements DataAlignerListener {
	private Progress progBar;
	private Map<Transition, XEventClass> activityMapping;
	private Map<String, String> variableMapping;

	@Plugin(name = "Conformance Checking of Petri Net With Data",
			parameterLabels = { "Petri Net With Data", "Log"},
			returnLabels = { "Petri Net with Data"},
			returnTypes = { ResultReplayPetriNetWithData.class },
			userAccessible = true,
			categories = { PluginCategory.ConformanceChecking },
			help = "Petri Net Data Alignment")
			@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "M. de Leoni", email = "m.d.leoni@tue.nl", pack = "DataAwareReplayer")
			public ResultReplayPetriNetWithData plugin(UIPluginContext context, DataPetriNet net, XLog log) 
	{
		XLogInfo summary = XLogInfoFactory.createLogInfo(log);
		
		EvClassLogPetrinetConnection conn;
		try {	
		conn = context.getConnectionManager().getFirstConnection(EvClassLogPetrinetConnection.class, context, net,
				log);
	} catch (Exception e) {
		JOptionPane.showMessageDialog(new JPanel(), "No mapping can be constructed between the net and the log");
		context.getFutureResult(0).cancel(true);
		return null;
	}
	
		// init gui for each step
		TransEvClassMapping activityMapping = (TransEvClassMapping) conn.getObjectWithRole(EvClassLogPetrinetConnection.TRANS2EVCLASSMAPPING);		
		HashMap<ReplayableTransition,XEventClass> activityMapping2=new HashMap<ReplayableTransition, XEventClass>();
		for(Entry<Transition, XEventClass> entry : activityMapping.entrySet())
			activityMapping2.put(new ReplayableTransition(entry.getKey()), entry.getValue());

		CostBasedCompleteParam parameters;
		try {
			parameters = createParameters(log, activityMapping, net, context);
		} catch (ConnectionCannotBeObtained e) {
			e.printStackTrace();
			return null;
		}		
		
		//Variable Mapping
		Collection<String> logAttributes=summary.getEventAttributeInfo().getAttributeKeys();
		TreeSet<String> processAttributes=new TreeSet<String>();
		for(DataElement variable : net.getVariables())
		{
			processAttributes.add(variable.getVarName());
		}

		MappingPanel<String, String> mapVariablePanel=new MappingPanel<String, String>(processAttributes,logAttributes);
		InteractionResult result = context.showConfiguration("Setup Variable Mapping", mapVariablePanel);
		if (result == InteractionResult.CANCEL) {
			context.getFutureResult(0).cancel(true);
			return null;
		}
		variableMapping=mapVariablePanel.getMapping(false);	
		//Variable Costs
		VariableMatchCostPanel<XEventClass> variablePanel=new VariableMatchCostPanel<XEventClass>(activityMapping2,mapVariablePanel.getMapping(true));
		result = context.showConfiguration("Variable Mapping Cost", variablePanel);
		if (result == InteractionResult.CANCEL) {
			context.getFutureResult(0).cancel(true);
			return null;
		}
		VariableMatchCosts variableCost=variablePanel.getCosts();	

		progBar = context.getProgress();
		progBar.setMinimum(0);
		progBar.setMaximum(log.size());
		progBar.setValue(0);
		return noInterface(context,log,net,activityMapping,variableMapping,variableCost,parameters,progBar);
	}
	
	
	public ResultReplayPetriNetWithData noInterface(UIPluginContext context, XLog log,DataPetriNet net,TransEvClassMapping activityMapping,
			Map<String,String> variableMapping,VariableMatchCosts variableCost,CostBasedCompleteParam parameters,Progress progBar)
	{	
		PNRepResult input = null;	
		this.progBar=progBar; 

		//Build Objects to call the alignment builder
		Map<String, Class> varType=new HashMap<String, Class>();
		Map<String, Object> upperBounds=new HashMap<String, Object>();
		Map<String, Object> lowerBounds=new HashMap<String, Object>();
		
		for(DataElement elem : net.getVariables())
		{
			varType.put(elem.getVarName(),elem.getType());
			Object value=elem.getMinValue();
			if (value!=null)
				lowerBounds.put(elem.getVarName(), value);
			value=elem.getMaxValue();
			if (value!=null)
				upperBounds.put(elem.getVarName(), value);			
		}
		
		ControlFlowAlignmentConnection connection;
		long time1=System.currentTimeMillis();
		long time2;	
		try {
			if (context!=null)
			{
				connection = context.getConnectionManager().getFirstConnection(ControlFlowAlignmentConnection.class, context, net, log);
				input=connection.getObjectWithRole(ControlFlowAlignmentConnection.PNREPRESULT);
			}
			else
			{
				try {
					input=new PNLogReplayer().replayLog(context, net, log, activityMapping, new PetrinetReplayerWithoutILP(), parameters);
				} catch (AStarException e) {
					e.printStackTrace();
				}				
			}
		} catch (ConnectionCannotBeObtained e1) {
			try {	
				input=new PNLogReplayer().replayLog(context, net, log, activityMapping, new PetrinetReplayerWithoutILP(), parameters);
				time1=System.currentTimeMillis()-time1;
				context.getConnectionManager().addConnection(new ControlFlowAlignmentConnection("Control-Flow Alignment Connection",net,log,input));
			} catch (AStarException e) {
				e.printStackTrace();
			}
		}		
		
		ControlFlowAlignmentBridge controlFlowAlignmentFactory = new ControlFlowAlignmentBridge(log, variableMapping, varType, activityMapping);
		Collection<ControlFlowAlignment> controlFlowAlignments=new ArrayList<ControlFlowAlignment>();
		for(SyncReplayResult alignment : input)
			for(Integer index : alignment.getTraceIndex())
			{
				float controlFlowFitness=alignment.getInfo().get(PNRepResult.TRACEFITNESS).floatValue();
				controlFlowAlignments.add((ControlFlowAlignment) controlFlowAlignmentFactory.buildControlFlowAlignment(alignment.getNodeInstance(), 
						alignment.getStepTypes(), log.get(index), index, controlFlowFitness));
			}
		
		time2=System.currentTimeMillis();
		
		Collection<DataAlignmentState> alignments=
				DataAlignerBuilder.buildAlignments(controlFlowAlignments, variableCost, varType, upperBounds, lowerBounds, this);
		
		System.out.println(System.currentTimeMillis()-time2+time1);
		
		return new ResultReplayPetriNetWithData(alignments, variableCost, variableMapping, net, log, activityMapping.getEventClassifier());

	}
	
	private static TransEvClassMapping createMapping(UIPluginContext context, PetrinetGraph net, XLog log) throws ConnectionCannotBeObtained {
		EvClassLogPetrinetConnection connection2=context.getConnectionManager().getFirstConnection(EvClassLogPetrinetConnection.class, context, net, log);
		TransEvClassMapping mapping = connection2.getObjectWithRole(EvClassLogPetrinetConnection.TRANS2EVCLASSMAPPING);
		Set<Transition> unmappedTrans = new HashSet<Transition>();
		for (Entry<Transition, XEventClass> entry : mapping.entrySet()) {
			if (entry.getValue().equals(mapping.getDummyEventClass())) {
				if (!entry.getKey().isInvisible()) {
					unmappedTrans.add(entry.getKey());
				}
			}
		}
		
		if (!unmappedTrans.isEmpty()) {
			JList list = new JList(unmappedTrans.toArray());
			JPanel panel = new JPanel();
			
			BoxLayout layout = new BoxLayout(panel, BoxLayout.Y_AXIS);
			panel.setLayout(layout);
			panel.add(new JLabel("The following transitions are not mapped to any event class:"));

			JScrollPane sp = new JScrollPane(list);
			panel.add(sp);
			panel.add(new JLabel("Do you want to consider these transitions as invisible (unlogged activities)?"));

			Object[] options = { "Yes, set them to invisible", "No, keep them as they are" };

			if (0 == JOptionPane.showOptionDialog(null, panel, "Configure transition visibility",
					JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, null, options, options[0])) {
				for (Transition t : unmappedTrans) {
					t.setInvisible(true);
				}
			}
			;
		}
		return mapping;
	}	
	
	/**
	 * Creates a marking of connection-type classType (i.e. FinalMarkingConnection.class)
	 * @param context
	 * @param net
	 * @param title
	 * @return
	 */
	private static Marking createMarking(UIPluginContext context, PetrinetGraph net, String title) {
        MarkingEditorPanel editor = new MarkingEditorPanel(title);

        return editor.getMarking(context, net);
	}		

	private static CostBasedCompleteParam createParameters(XLog log, TransEvClassMapping activityMapping, PetrinetGraph net, PluginContext context) throws ConnectionCannotBeObtained
	{
		XLogInfo logInfo=XLogInfoFactory.createLogInfo(log, activityMapping.getEventClassifier());
		CostBasedCompleteParam parameter = new CostBasedCompleteParam(logInfo.getEventClasses().getClasses(),
				activityMapping.getDummyEventClass(), net.getTransitions(), 2, 5);
		parameter.setGUIMode(false);
		parameter.setCreateConn(false);
		
		Marking initMarking = null;
		try {
			initMarking = context.getConnectionManager().getFirstConnection(InitialMarkingConnection.class, context, net).
					getObjectWithRole(FinalMarkingConnection.MARKING);
		} catch (Exception e) {
				initMarking=createMarking((UIPluginContext) context, net, "Initial Marking");
				context.getConnectionManager().addConnection(new InitialMarkingConnection(net, initMarking));
		}
		parameter.setInitialMarking(initMarking);
		
		Marking finalMarking = null;
		try {
			finalMarking = context.getConnectionManager().getFirstConnection(FinalMarkingConnection.class, context, net).
					getObjectWithRole(FinalMarkingConnection.MARKING);
		} catch (Exception e) {
				finalMarking=createMarking((UIPluginContext) context, net, "Final Marking");
				context.getConnectionManager().addConnection(new FinalMarkingConnection(net, finalMarking));
		}

		parameter.setFinalMarkings(new Marking[] {finalMarking});
		parameter.setMaxNumOfStates(200000);
		return parameter;
	}	
	
	public void exceptionThrow(String traceName,Exception err) {
		err.printStackTrace();
	}

	public void newAlignedTrace(DataAlignmentState da) {		
		synchronized(this)
		{
			if (progBar!=null)
				progBar.inc();
		}
	}

	public void startNewAlignment(String traceName) {		
	}

	public void exceptionThrow(Exception err) {
		err.printStackTrace();
	}

	public void alignmentImproved(DataAlignmentState da) {
	}

	public void alignmentCalculated(DataAlignmentState da) {		
	}

}