package org.processmining.plugins.DataConformance.visualization.grouping;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.concurrent.ExecutionException;

import javax.swing.JOptionPane;
import javax.swing.SwingWorker;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.util.ui.widgets.helper.ProMUIHelper;
import org.processmining.framework.util.ui.widgets.traceview.masterdetail.MasterView;
import org.processmining.log.utils.XUtils;
import org.processmining.plugins.DataConformance.visualization.alignment.XTraceResolver;
import org.processmining.plugins.DataConformance.visualization.grouping.GroupedAlignments.AlignmentGroup;
import org.processmining.plugins.utils.ProvidedObjectHelper;
import org.processmining.xesalignmentextension.XAlignmentExtension.XAlignment;

final class ActionExportOriginalTraces implements ActionListener {

	private final PluginContext context;
	private final MasterView<AlignmentGroup, XAlignment> masterView;
	private final XTraceResolver resolver;

	public ActionExportOriginalTraces(PluginContext context, XTraceResolver resolver,
			MasterView<AlignmentGroup, XAlignment> masterView) {
		this.context = context;
		this.resolver = resolver;
		this.masterView = masterView;
	}

	public void actionPerformed(ActionEvent e) {
		@SuppressWarnings("deprecation")
		final Object[] selection = masterView.getMasterList().getList().getSelectedValues();

		String proposedName = selection.length == 1 ? ((AlignmentGroup) selection[0]).getName() : "Group";
		final String groupName = JOptionPane.showInputDialog(masterView.getMasterList(),
				"Specify the name of the new ProM log!", proposedName);

		if (groupName != null) {

			masterView.getMasterList().beforeUpdate();

			new SwingWorker<XLog, Void>() {

				protected XLog doInBackground() throws Exception {
					XLog newLog = XFactoryRegistry.instance().currentDefault().createLog();
					XConceptExtension.instance().assignName(newLog, groupName);

					int groupIndex = 0;
					for (Object o : selection) {
						AlignmentGroup group = (AlignmentGroup) o;
						for (XAlignment a : masterView.getDetailElements(group)) {
							XTrace trace = resolver.getOriginalTrace(XUtils.getConceptName(a.getTrace()));
							if (trace != null) {
								XTrace clonedTrace = (XTrace) trace.clone();
								addMetaData(clonedTrace, groupIndex, group.size());
								newLog.add(clonedTrace);
							}
						}
						groupIndex++;
					}
					return newLog;
				}

				private void addMetaData(XTrace detailTraces, int groupIndex, int groupCount) {
					XUtils.putAttribute(detailTraces,
							XUtils.createAttribute("variant:index", Long.valueOf(groupIndex)));
					XUtils.putAttribute(detailTraces,
							XUtils.createAttribute("variant:count", Long.valueOf(groupCount)));
				}

				protected void done() {
					try {
						ProvidedObjectHelper.publish(context, groupName, get(), XLog.class, true);
					} catch (InterruptedException | ExecutionException e) {
						ProMUIHelper.showErrorMessage(masterView.getMasterComponent(), e.toString(),
								"Could not export traces");
					} finally {
						masterView.getMasterList().afterUpdate();
					}
				}

			}.execute();

		}
	}
}