package org.processmining.plugins.DataConformance.visualization.grouping;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.prefs.Preferences;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileNameExtensionFilter;

import org.processmining.framework.util.ui.widgets.helper.ProMUIHelper;
import org.processmining.framework.util.ui.widgets.traceview.ProMTraceList;
import org.processmining.framework.util.ui.widgets.traceview.masterdetail.MasterView;
import org.processmining.plugins.DataConformance.visualization.grouping.GroupedAlignments.AlignmentGroup;
import org.processmining.xesalignmentextension.XAlignmentExtension.XAlignment;

final class ActionListenerSaveAs implements ActionListener {
	
	private static final Preferences PREFS = Preferences.userRoot().node("org.processmining.dataawarereplayer");
	private static final String LAST_USED_FOLDER = "lastUsedFolder";

	private final MasterView<AlignmentGroup, XAlignment> masterView;
	private final Comparator<AlignmentGroup> currentOrdering;

	public ActionListenerSaveAs(MasterView<AlignmentGroup, XAlignment> masterView,
			Comparator<AlignmentGroup> currentOrdering) {
		super();
		this.masterView = masterView;
		this.currentOrdering = currentOrdering;
	}

	public void actionPerformed(ActionEvent e) {

		JFileChooser chooser = new JFileChooser(PREFS.get(LAST_USED_FOLDER, new File(".").getAbsolutePath()));
		FileNameExtensionFilter pdfFilter = new FileNameExtensionFilter("PDF", "pdf");
		FileNameExtensionFilter emfFilter = new FileNameExtensionFilter("EMF", "emf");
		FileNameExtensionFilter epsFilter = new FileNameExtensionFilter("EPS", "eps");
		chooser.addChoosableFileFilter(pdfFilter);
		chooser.addChoosableFileFilter(emfFilter);
		chooser.addChoosableFileFilter(epsFilter);
		chooser.setFileFilter(pdfFilter);
		chooser.setAcceptAllFileFilterUsed(false);
		int returnVal = chooser.showSaveDialog(masterView.getMasterComponent());
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			File file = chooser.getSelectedFile();
			PREFS.put(LAST_USED_FOLDER, file.getParent());
			
			List<AlignmentGroup> traces = new ArrayList<>();
			Object[] selection = masterView.getMasterList().getList().getSelectedValues();
			for (Object o : selection) {
				traces.add((AlignmentGroup) o);
			}

			ProMTraceList<AlignmentGroup> listForPrinting = new ProMTraceList<>(traces, masterView.getMasterList()
					.getTraceBuilder(), currentOrdering);

			listForPrinting.setOpaque(true);
			listForPrinting.setBackground(Color.WHITE);
			listForPrinting.setForeground(Color.BLACK);

			if (chooser.getFileFilter() == pdfFilter) {
				if (!file.getAbsolutePath().endsWith(".pdf")) {
					file = new File(file.getAbsolutePath() + ".pdf");
				}
				try {
					ProMTraceList.saveAsPDF(listForPrinting, "", file);
				} catch (IOException e1) {
					ProMUIHelper.showErrorMessage(masterView.getMasterComponent(), e1.getMessage(), "Error saving");
				}	
			} else if (chooser.getFileFilter() == emfFilter) {
				if (!file.getAbsolutePath().endsWith(".emf")) {
					file = new File(file.getAbsolutePath() + ".emf");
				}
				try {
					ProMTraceList.saveAsEMF(listForPrinting, file);
				} catch (IOException e1) {
					ProMUIHelper.showErrorMessage(masterView.getMasterComponent(), e1.getMessage(), "Error saving");
				}
			} else if (chooser.getFileFilter() == epsFilter) {
				if (!file.getAbsolutePath().endsWith(".eps")) {
					file = new File(file.getAbsolutePath() + ".eps");
				}
				try {
					ProMTraceList.saveAsEPS(listForPrinting, file);
				} catch (IOException e1) {
					ProMUIHelper.showErrorMessage(masterView.getMasterComponent(), e1.getMessage(), "Error saving");
				}
			}

		}

	}

}