package org.processmining.plugins.balancedconformance.controlflow.override;

import gnu.trove.list.TIntList;
import gnu.trove.list.TShortList;
import gnu.trove.list.array.TIntArrayList;
import nl.tue.astar.Delegate;
import nl.tue.astar.Head;
import nl.tue.astar.Record;
import nl.tue.astar.Tail;
import nl.tue.astar.util.ShortShortMultiset;
import nl.tue.storage.hashing.HashCodeProvider;

import org.deckfour.xes.model.XTrace;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.astar.petrinet.impl.AbstractPDelegate;
import org.processmining.plugins.astar.petrinet.impl.PHead;

public final class DHead extends PHead {

	public DHead(AbstractPDelegate<?> delegate, Marking m, XTrace t) {
		super(delegate, m, t);
	}

	public DHead(ShortShortMultiset marking, ShortShortMultiset parikh, int hashCode) {
		super(marking, parikh, hashCode);
	}

	static HashCodeProvider getHashCodeProvider() {
		return PROVIDER;
	}

	@Override
	protected DHead createHead(ShortShortMultiset marking, ShortShortMultiset parikh, int hashCode) {
		return new DHead(marking, parikh, hashCode);
	}

	@Override
	protected ShortShortMultiset cloneAndUpdateMarking(AbstractPDelegate<?> delegate, ShortShortMultiset marking,
			short modelMove) {
		// clone the marking		
		ShortShortMultiset newMarking = marking.clone();
		// update marking
		updateIncoming(delegate.getInputOf(modelMove), newMarking);
		updateOutgoing(delegate.getOutputOf(modelMove), newMarking);
		return newMarking;
	}

	private static void updateOutgoing(short[] out, ShortShortMultiset newMarking) {
		for (int place = out.length; place-- > 0;) {
			short produced = out[place];
			if (produced != 0) {
				if (produced < 0) {
					produced = adjustForResetArcs(newMarking, (short) place, produced);
				}
				newMarking.adjustValue((short) place, produced);
			}
		}
	}

	private static void updateIncoming(short[] in, ShortShortMultiset newMarking) {
		for (int place = in.length; place-- > 0;) {
			short needed = in[place];
			if (needed > 0) { // implies "needed != AbstractPDelegate.INHIBITED"
				// only adjust the value for non-inhibitor arcs
				newMarking.adjustValue((short) place, (short) -needed);
			}
		}
	}

	private static short adjustForResetArcs(ShortShortMultiset newMarking, short place, short produced) {
		// combination or reset arc and regular arc (regular arc may be 0)
		// first get the actual produced tokens
		produced = (short) (-(produced + 1));
		// then account for removing all tokens first
		produced -= newMarking.get(place);
		return produced;
	}

	public TIntList getSynchronousMoves(Record rec, Delegate<? extends Head, ? extends Tail> d, TIntList enabled,
			int activity) {
		final AbstractPDelegate<?> delegate = (AbstractPDelegate<?>) d;

		// only consider transitions mapped to activity
		final TIntList result = new TIntArrayList();
		TShortList mappedTrans = delegate.getTransitions((short) activity);
		// Performance: use supplied list of enabled transitions
		for (int i = 0; i < mappedTrans.size(); i++) {
			short mappedIndex = mappedTrans.get(i);
			// Enabled is sorted in ascending order
			if (enabled.contains(mappedIndex)) {
				result.add(mappedIndex);
			}
		}

		return result;
	}

}