/*
 * Copyright (c) 2013 F. Mannhardt (f.mannhardt@tue.nl)
 */
package org.processmining.plugins.balancedconformance.dataflow;

import org.processmining.plugins.balancedconformance.dataflow.datastate.DataState;

/**
 * Stores the result of the data alignment for a prefix.
 * 
 * @author F. Mannhardt
 * 
 */
public class PartialDataAlignedTrace {

	public static final PartialDataAlignedTrace NO_RESULT = new PartialDataAlignedTrace(Integer.MAX_VALUE, null);
	public static final PartialDataAlignedTrace IMPOSSIBLE = new PartialDataAlignedTrace(Integer.MAX_VALUE, null);

	private final int dataCost;
	private final DataState dataState;
	private final int dataStateIndex;

	public PartialDataAlignedTrace(int dataCost, DataState dataState) {
		this.dataCost = dataCost;
		this.dataState = dataState;
		this.dataStateIndex = -1;
	}

	public PartialDataAlignedTrace(int f_cost, int dataStateIndex) {
		this.dataCost = f_cost;
		this.dataState = null;
		this.dataStateIndex = dataStateIndex;
	}

	public int getDataCost() {
		return dataCost;
	}

	public DataState getVariableAssignment() {
		return dataState;
	}

	public int getDataStateIndex() {
		return dataStateIndex;
	}

	public String toString() {
		return String.format("PartialDataAlignedTrace [dataCost=%s, dataState=%s, dataStateIndex=%s]", dataCost,
				dataState, dataStateIndex);
	}

}
