/*
 * Copyright (c) 2013 F. Mannhardt (f.mannhardt@tue.nl)
 */
package org.processmining.plugins.balancedconformance.dataflow.datastate;

import java.util.HashMap;
import java.util.Map;

/**
 * 
 * @author F. Mannhardt
 * 
 */
public final class DataStateStoreJavaNoLockImpl implements DataStateStore {

	private final Map<DataState, Integer> dataStateStore;
	private final Map<Integer, DataState> reverseDataStateStore;
	private int keyCounter = START_INDEX;

	public DataStateStoreJavaNoLockImpl(int initialCapacity) {
		super();
		dataStateStore = new HashMap<>(initialCapacity);
		reverseDataStateStore = new HashMap<>(initialCapacity);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.processmining.plugins.balancedconformance.dataflow.DataStateStore
	 * #get(long)
	 */
	public DataState get(int index) {
		if (index == 0) {
			return EMPTY_IMMUTABLE_STATE;
		}
		return reverseDataStateStore.get(index);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.processmining.plugins.balancedconformance.dataflow.datastate.
	 * DataStateStore
	 * #putIfAbsent(org.processmining.plugins.balancedconformance.dataflow
	 * .datastate.DataState)
	 */
	public int putIfAbsent(DataState state) {
		if (state == EMPTY_IMMUTABLE_STATE) {
			return EMPTY_STATE_INDEX;
		}
		state = state.makeStateImmutable();
		Integer idx = dataStateStore.get(state);
		if (idx != null) {
			return idx;
		}
		int stateId = keyCounter++;
		dataStateStore.put(state, stateId);
		reverseDataStateStore.put(stateId, state);
		return stateId;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.processmining.plugins.balancedconformance.dataflow.DataStateStore
	 * #size()
	 */
	public int size() {
		return dataStateStore.size();
	}

}
