package org.processmining.plugins.balancedconformance.functions.impl;

import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XEvent;
import org.processmining.datapetrinets.expression.syntax.ExprFunction;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.DataConformance.framework.WriteOperationCost;
import org.processmining.plugins.balancedconformance.dataflow.datastate.DataStateHelper;
import org.processmining.plugins.balancedconformance.functions.FunctionTernary;
import org.processmining.plugins.balancedconformance.functions.VirtualVariable;
import org.processmining.plugins.balancedconformance.mapping.LogMapping;
import org.processmining.plugins.balancedconformance.mapping.Variable;

import com.google.common.collect.ImmutableMap;
import com.google.common.collect.ImmutableSetMultimap;
import com.google.common.collect.SetMultimap;

final class StringContainsFunction implements FunctionTernary {

	final static class StringContainsVariableImpl extends StringFunctionVariable {

		public StringContainsVariableImpl(ImmutableMap<String, Variable> normalVariables, WriteOperationCost writeOperationCost, Transition transition,
				String variableName, String parameterValue, String optionalIdentifier) {
			super(normalVariables, writeOperationCost, transition, variableName, parameterValue, optionalIdentifier);
		}

		public Object compute(Prefix alignmentPrefix, Transition currentTransition, XEvent currentEvent,
				LogMapping mapping) {

			if (isPrime) {
				if (currentEvent != null) {
					XAttribute attribute = currentEvent.getAttributes().get(getTargetAttributeName());
					if (attribute instanceof XAttributeLiteral) {
						String variableValue = ((XAttributeLiteral) attribute).getValue();
						String lowerCaseValue = variableValue.toLowerCase();
						return lowerCaseValue.contains(value.toLowerCase());
					} else {
						// Missing attribute
						return null;
					}
				} else {
					return null;
				}
			} else {
				Object variableValue = alignmentPrefix.variableValue(getTargetAttributeName());
				if (variableValue != null) {
					String lowerCaseValue = ((String) variableValue).toLowerCase();
					return lowerCaseValue.contains(value.toLowerCase());
				} else {
					return null;
				}
			}
		}

		protected String determineName(String variableName, String identifier) {
			return DataStateHelper.stripPrimeIfNeeded(variableName) + "Contains" + identifier;
		}

	}

	public boolean isValidFunction(ExprFunction node) {
		return node.jjtGetNumChildren() >= 2;
	}

	public VirtualVariable createPrimaryVariable(ImmutableMap<String, Variable> normalVariables,
			WriteOperationCost writeOperationCost, SetMultimap<String, Transition> nameToTransition,
			Transition transition, String parameter1, String parameter2, String parameter3) {
		return new StringContainsVariableImpl(normalVariables, writeOperationCost, transition, parameter1, parameter2, parameter3);
	}

	public SetMultimap<Transition, VirtualVariable> createHelperVariables(
			ImmutableMap<String, Variable> normalVariables, WriteOperationCost writeOperationCost,
			SetMultimap<String, Transition> nameToTransition, Transition transition,
			VirtualVariable primaryVariable, String parameter1, String parameter2, String parameter3) {
		return ImmutableSetMultimap.of();
	}

}