package org.processmining.plugins.balancedconformance.result;

import java.util.Iterator;

import org.processmining.plugins.DataConformance.framework.ExecutionStep;
import org.processmining.plugins.DataConformance.framework.ExecutionTrace;

public class DataAlignedTraceImpl implements DataAlignedTrace {

	private final ExecutionTrace logTrace;
	private final ExecutionTrace processTrace;
	private final float controlFlowCost;
	private final int dataCost;

	public DataAlignedTraceImpl(ExecutionTrace logTrace, ExecutionTrace processTrace, float controlFlowCost, int dataCost) {
		this.logTrace = logTrace;
		this.processTrace = processTrace;
		this.controlFlowCost = controlFlowCost;
		this.dataCost = dataCost;
	}

	@Override
	public int getOverallCost() {
		//TODO should be both int!!
		return (int)controlFlowCost + dataCost;
	}

	@Override
	public int getDataCost() {
		return dataCost;
	}

	@Override
	public ExecutionTrace getLogTrace() {
		return logTrace;
	}

	@Override
	public ExecutionTrace getProcessTrace() {
		return processTrace;
	}
	
	@Override
	public String toString() {
		StringBuffer buffer = new StringBuffer();

		Iterator<ExecutionStep> moveInProcessIter = processTrace.iterator();
		Iterator<ExecutionStep> moveInLogIter = logTrace.iterator();

		buffer.append("[");

		while (moveInLogIter.hasNext() && moveInProcessIter.hasNext()) {
			ExecutionStep logStep = moveInLogIter.next();
			ExecutionStep processStep = moveInProcessIter.next();

			if (processStep.isInvisible()) {
				if (logStep.equalsAssignmentsOnly(processStep)) {
					buffer.append(processStep.getActivity() + "(INV)");
				} else {
					buffer.append(processStep.getActivity() + "(INV - DATA)");
				}
			} else {
				if (logStep.getActivity() != null && processStep.getActivity() != null) {
					//Move in Both
					if (logStep.equalsAssignmentsOnly(processStep)) {
						buffer.append(logStep.getActivity() + "(GOOD)");	
					} else {
						buffer.append(logStep.getActivity() + "(DATA)");
					}					
				} else if (logStep.getActivity() != null) {
					buffer.append(logStep.getActivity() + "(LOG)");
				} else if (processStep.getActivity() != null) {
					buffer.append(processStep.getActivity() + "(MODEL)");
				}
			}

			if (moveInLogIter.hasNext() && moveInProcessIter.hasNext()) {
				buffer.append(", ");
			}
		}

		buffer.append("]");
		
		buffer.append(" Cost: "+getOverallCost());

		return buffer.toString();
	}

}
